@@ -23,9 +23,9 @@
 
 #include <algorithm>
 #include <functional>
-#include <tr1/unordered_map>
-using std::tr1::unordered_map;
-using std::tr1::unordered_multimap;
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
 #include <vector>
 using std::vector;
 
@@ -47,7 +47,7 @@ class DefaultAccumulator {
 
   DefaultAccumulator() {}
 
-  DefaultAccumulator(const DefaultAccumulator<A> &acc) {}
+  DefaultAccumulator(const DefaultAccumulator<A> &acc, bool safe = false) {}
 
   void Init(const Fst<A>& fst, bool copy = false) {}
 
@@ -86,7 +86,7 @@ class LogAccumulator {
 
   LogAccumulator() {}
 
-  LogAccumulator(const LogAccumulator<A> &acc) {}
+  LogAccumulator(const LogAccumulator<A> &acc, bool safe = false) {}
 
   void Init(const Fst<A>& fst, bool copy = false) {}
 
@@ -134,7 +134,7 @@ class FastLogAccumulatorData {
 
   vector<double> *Weights() { return &weights_; }
   vector<ssize_t> *WeightPositions() { return &weight_positions_; }
-  double *WeightEnd() { return &(weights_[weights_.size() - 1]); };
+  double *WeightEnd() { return &(weights_[weights_.size() - 1]); }
   int RefCount() const { return ref_count_.count(); }
   int IncrRefCount() { return ref_count_.Incr(); }
   int DecrRefCount() { return ref_count_.Decr(); }
@@ -171,7 +171,7 @@ class FastLogAccumulator {
         data_(new FastLogAccumulatorData()),
         error_(false) {}
 
-  FastLogAccumulator(const FastLogAccumulator<A> &acc)
+  FastLogAccumulator(const FastLogAccumulator<A> &acc, bool safe = false)
       : arc_limit_(acc.arc_limit_),
         arc_period_(acc.arc_period_),
         data_(acc.data_),
@@ -349,6 +349,11 @@ class CacheLogAccumulatorData {
   CacheLogAccumulatorData(bool gc, size_t gc_limit)
       : cache_gc_(gc), cache_limit_(gc_limit), cache_size_(0) {}
 
+  CacheLogAccumulatorData(const CacheLogAccumulatorData<A> &data)
+      : cache_gc_(data.cache_gc_),
+        cache_limit_(data.cache_limit_),
+        cache_size_(0) {}
+
   ~CacheLogAccumulatorData() {
     for(typename unordered_map<StateId, CacheState>::iterator it = cache_.begin();
         it != cache_.end();
@@ -417,7 +422,7 @@ class CacheLogAccumulatorData {
   size_t cache_size_;                    // # of bytes allowed before GC
   RefCounter ref_count_;
 
-  DISALLOW_COPY_AND_ASSIGN(CacheLogAccumulatorData);
+  void operator=(const CacheLogAccumulatorData<A> &);   // Disallow
 };
 
 // This class accumulates arc weights using the log semiring Plus()
@@ -439,10 +444,11 @@ class CacheLogAccumulator {
           new CacheLogAccumulatorData<A>(gc, gc_limit)), s_(kNoStateId),
         error_(false) {}
 
-  CacheLogAccumulator(const CacheLogAccumulator<A> &acc)
+  CacheLogAccumulator(const CacheLogAccumulator<A> &acc, bool safe = false)
       : arc_limit_(acc.arc_limit_), fst_(acc.fst_ ? acc.fst_->Copy() : 0),
-        data_(acc.data_), s_(kNoStateId), error_(acc.error_) {
-    data_->IncrRefCount();
+        data_(safe ? new CacheLogAccumulatorData<A>(*acc.data_) : acc.data_),
+        s_(kNoStateId), error_(acc.error_) {
+    if (!safe) data_->IncrRefCount();
   }
 
   ~CacheLogAccumulator() {
@@ -504,12 +510,14 @@ class CacheLogAccumulator {
         sum = LogPlus(sum, aiter->Value().weight);
       return sum;
     } else {
-      if (weights_->size() <= end)
+      if (weights_->size() <= end) {
         for (aiter->Seek(weights_->size() - 1);
              weights_->size() <= end;
-             aiter->Next())
+             aiter->Next()) {
           weights_->push_back(LogPlus(weights_->back(),
                                       aiter->Value().weight));
+        }
+      }
       return LogPlus(w, LogMinus((*weights_)[end], (*weights_)[begin]));
     }
   }
@@ -619,9 +627,10 @@ class ReplaceAccumulatorData {
     state_table_ = state_table;
     accumulators_.resize(fst_tuples.size());
     for (size_t i = 0; i < accumulators_.size(); ++i) {
-      if (!accumulators_[i])
+      if (!accumulators_[i]) {
         accumulators_[i] = new Accumulator;
-      accumulators_[i]->Init(*(fst_tuples[i].second));
+        accumulators_[i]->Init(*(fst_tuples[i].second));
+      }
       fst_array_.push_back(fst_tuples[i].second->Copy());
     }
   }
@@ -664,23 +673,30 @@ class ReplaceAccumulator {
 
   ReplaceAccumulator()
       : init_(false), data_(new ReplaceAccumulatorData<Accumulator, T>()),
-        error_(false) {}
+        aiter_(0), error_(false) {}
 
   ReplaceAccumulator(const vector<Accumulator*> &accumulators)
       : init_(false),
         data_(new ReplaceAccumulatorData<Accumulator, T>(accumulators)),
-        error_(false) {}
+        aiter_(0), error_(false) {}
 
-  ReplaceAccumulator(const ReplaceAccumulator<Accumulator, T> &acc)
-      : init_(acc.init_), data_(acc.data_), error_(acc.error_) {
-    if (!init_)
+  ReplaceAccumulator(const ReplaceAccumulator<Accumulator, T> &acc,
+                     bool safe = false)
+      : init_(acc.init_), data_(acc.data_), aiter_(0), error_(acc.error_) {
+    if (!init_) {
       FSTERROR() << "ReplaceAccumulator: can't copy unintialized accumulator";
+    }
+    if (safe) {
+      FSTERROR() << "ReplaceAccumulator: safe copy not supported";
+    }
     data_->IncrRefCount();
   }
 
   ~ReplaceAccumulator() {
      if (!data_->DecrRefCount())
       delete data_;
+     if (aiter_)
+       delete aiter_;
   }
 
   // Does not take ownership of the state table, the state table
@@ -691,6 +707,20 @@ class ReplaceAccumulator {
     data_->Init(fst_tuples, state_table);
   }
 
+  // Method required by LookAheadMatcher. However, ReplaceAccumulator
+  // needs to be initialized by calling the Init method above before
+  // being passed to LookAheadMatcher.
+  // TODO(allauzen): Revisit this. Consider creating an
+  // Init(const ReplaceFst<A, T, C>&, bool) method and using friendship to
+  // get access to the innards of ReplaceFst.
+  void Init(const Fst<Arc> &fst, bool copy = false) {
+    if (!init_) {
+      FSTERROR() << "ReplaceAccumulator::Init: accumulator needs to be"
+                 << " initialized before being passed to LookAheadMatcher.";
+      error_ = true;
+    }
+  }
+
   void SetState(StateId s) {
     if (!init_) {
       FSTERROR() << "ReplaceAccumulator::SetState: incorrectly initialized.";
@@ -708,6 +738,9 @@ class ReplaceAccumulator {
       offset_ = 0;
       offset_weight_ = Weight::Zero();
     }
+    if (aiter_) delete aiter_;
+    aiter_ = new ArcIterator<Fst<Arc> >(
+        *data_->GetFst(fst_id_), tuple.fst_state);
   }
 
   Weight Sum(Weight w, Weight v) {
@@ -721,7 +754,7 @@ class ReplaceAccumulator {
     if (error_) return Weight::NoWeight();
     Weight sum = begin == end ? Weight::Zero()
         : data_->GetAccumulator(fst_id_)->Sum(
-            w, aiter, begin ? begin - offset_ : 0, end - offset_);
+            w, aiter_, begin ? begin - offset_ : 0, end - offset_);
     if (begin == 0 && end != 0 && offset_ > 0)
       sum = Sum(offset_weight_, sum);
     return sum;
@@ -735,6 +768,7 @@ class ReplaceAccumulator {
   Label fst_id_;
   size_t offset_;
   Weight offset_weight_;
+  ArcIterator<Fst<Arc> > *aiter_;
   bool error_;
 
   void operator=(const ReplaceAccumulator<Accumulator, T> &);   // Disallow
