@@ -28,6 +28,7 @@ using std::vector;
 #include <fst/union-find.h>
 #include <fst/mutable-fst.h>
 
+
 namespace fst {
 
 // Finds and returns connected components. Use with Visit().
@@ -40,57 +41,75 @@ class CcVisitor {
 
   // cc[i]: connected component number for state i.
   CcVisitor(vector<StateId> *cc)
-  : cc_(cc),
-    nstates_(0),
-    comps_(0, kNoStateId) {
-    cc_->clear();
+      : comps_(new UnionFind<StateId>(0, kNoStateId)),
+        cc_(cc),
+        nstates_(0) { }
+
+  // comps: connected components equiv classes.
+  CcVisitor(UnionFind<StateId> *comps)
+      : comps_(comps),
+        cc_(0),
+        nstates_(0) { }
+
+  ~CcVisitor() {
+    if (cc_)  // own comps_?
+      delete comps_;
   }
 
-  void InitVisit(const Fst<A> &fst) {
-    CHECK(cc_->empty());
-  }
+  void InitVisit(const Fst<A> &fst) { }
 
   bool InitState(StateId s, StateId root) {
     ++nstates_;
-    if (comps_.FindSet(s) == kNoStateId)
-      comps_.MakeSet(s);
+    if (comps_->FindSet(s) == kNoStateId)
+      comps_->MakeSet(s);
     return true;
   }
 
   bool WhiteArc(StateId s, const A &arc) {
-    comps_.MakeSet(arc.nextstate);
-    comps_.Union(s, arc.nextstate);
+    comps_->MakeSet(arc.nextstate);
+    comps_->Union(s, arc.nextstate);
     return true;
   }
 
   bool GreyArc(StateId s, const A &arc) {
-    comps_.Union(s, arc.nextstate);
+    comps_->Union(s, arc.nextstate);
     return true;
   }
 
   bool BlackArc(StateId s, const A &arc) {
-    comps_.Union(s, arc.nextstate);
+    comps_->Union(s, arc.nextstate);
     return true;
   }
 
   void FinishState(StateId s) { }
 
   void FinishVisit() {
+    if (cc_)
+      GetCcVector(cc_);
+  }
+
+  // cc[i]: connected component number for state i.
+  // Returns number of components.
+  int GetCcVector(vector<StateId> *cc) {
+    cc->clear();
+    cc->resize(nstates_, kNoStateId);
     StateId ncomp = 0;
-    vector<StateId> rep2comp(nstates_, kNoStateId);
-    for (int i = 0; i < nstates_; ++i) {
-      StateId rep = comps_.FindSet(i);
-      StateId &comp = rep2comp[rep];
-      if (comp == kNoStateId)
-        comp = ncomp++;
-      cc_->push_back(comp);
+    for (StateId i = 0; i < nstates_; ++i) {
+      StateId rep = comps_->FindSet(i);
+      StateId &comp = (*cc)[rep];
+      if (comp == kNoStateId) {
+        comp = ncomp;
+        ++ncomp;
+      }
+      (*cc)[i] = comp;
     }
+    return ncomp;
   }
 
  private:
+  UnionFind<StateId> *comps_;   // Components
   vector<StateId> *cc_;         // State's cc number
   StateId nstates_;             // State count
-  UnionFind<StateId> comps_;    // Components
 };
 
 
@@ -102,7 +121,7 @@ template <class A>
 class SccVisitor {
  public:
   typedef A Arc;
-  typedef typename Arc::Weight Weight;
+  typedef typename A::Weight Weight;
   typedef typename A::StateId StateId;
 
   // scc[i]: strongly-connected component number for state i.
@@ -111,64 +130,16 @@ class SccVisitor {
   // coaccess[i]: coaccessibility of state i.
   // Any of above can be NULL.
   // props: related property bits (cyclicity, initial cyclicity,
-  //   accessibiliity, coaccessiblity) set/cleared (o.w. unchanged).
+  //   accessibility, coaccessibility) set/cleared (o.w. unchanged).
   SccVisitor(vector<StateId> *scc, vector<bool> *access,
              vector<bool> *coaccess, uint64 *props)
       : scc_(scc), access_(access), coaccess_(coaccess), props_(props) {}
   SccVisitor(uint64 *props)
       : scc_(0), access_(0), coaccess_(0), props_(props) {}
 
-  void InitVisit(const Fst<A> &fst) {
-    if (scc_)
-      scc_->clear();
-    if (access_)
-      access_->clear();
-    if (coaccess_) {
-      coaccess_->clear();
-      coaccess_internal_ = false;
-    } else {
-      coaccess_ = new vector<bool>;
-      coaccess_internal_ = true;
-    }
-    *props_ |= kAcyclic | kInitialAcyclic | kAccessible | kCoAccessible;
-    *props_ &= ~(kCyclic | kInitialCyclic | kNotAccessible | kNotCoAccessible);
-    fst_ = &fst;
-    start_ = fst.Start();
-    nstates_ = 0;
-    nscc_ = 0;
-    dfnumber_ = new vector<StateId>;
-    lowlink_ = new vector<StateId>;
-    onstack_ = new vector<bool>;
-    scc_stack_ = new vector<StateId>;
-  }
+  void InitVisit(const Fst<A> &fst);
 
-  bool InitState(StateId s, StateId root) {
-    scc_stack_->push_back(s);
-    while (dfnumber_->size() <= s) {
-      if (scc_)
-        scc_->push_back(-1);
-      if (access_)
-        access_->push_back(false);
-      coaccess_->push_back(false);
-      dfnumber_->push_back(-1);
-      lowlink_->push_back(-1);
-      onstack_->push_back(false);
-    }
-    (*dfnumber_)[s] = nstates_;
-    (*lowlink_)[s] = nstates_;
-    (*onstack_)[s] = true;
-    if (root == start_) {
-      if (access_)
-        (*access_)[s] = true;
-    } else {
-      if (access_)
-        (*access_)[s] = false;
-      *props_ |= kNotAccessible;
-      *props_ &= ~kAccessible;
-    }
-    ++nstates_;
-    return true;
-  }
+  bool InitState(StateId s, StateId root);
 
   bool TreeArc(StateId s, const A &arc) { return true; }
 
@@ -197,40 +168,7 @@ class SccVisitor {
     return true;
   }
 
-  void FinishState(StateId s, StateId p, const A *) {
-    if (fst_->Final(s) != Weight::Zero())
-      (*coaccess_)[s] = true;
-    if ((*dfnumber_)[s] == (*lowlink_)[s]) {  // root of new SCC
-      bool scc_coaccess = false;
-      size_t i = scc_stack_->size();
-      StateId t;
-      do {
-        t = (*scc_stack_)[--i];
-        if ((*coaccess_)[t])
-          scc_coaccess = true;
-      } while (s != t);
-      do {
-        t = scc_stack_->back();
-        if (scc_)
-          (*scc_)[t] = nscc_;
-        if (scc_coaccess)
-          (*coaccess_)[t] = true;
-        (*onstack_)[t] = false;
-        scc_stack_->pop_back();
-      } while (s != t);
-      if (!scc_coaccess) {
-        *props_ |= kNotCoAccessible;
-        *props_ &= ~kCoAccessible;
-      }
-      ++nscc_;
-    }
-    if (p != kNoStateId) {
-      if ((*coaccess_)[s])
-        (*coaccess_)[p] = true;
-      if ((*lowlink_)[s] < (*lowlink_)[p])
-        (*lowlink_)[p] = (*lowlink_)[s];
-    }
-  }
+  void FinishState(StateId s, StateId p, const A *);
 
   void FinishVisit() {
     // Numbers SCC's in topological order when acyclic.
@@ -261,6 +199,97 @@ class SccVisitor {
   vector<StateId> *scc_stack_;  // SCC stack (w/ random access)
 };
 
+template <class A> inline
+void SccVisitor<A>::InitVisit(const Fst<A> &fst) {
+  if (scc_)
+    scc_->clear();
+  if (access_)
+    access_->clear();
+  if (coaccess_) {
+    coaccess_->clear();
+    coaccess_internal_ = false;
+  } else {
+    coaccess_ = new vector<bool>;
+    coaccess_internal_ = true;
+  }
+  *props_ |= kAcyclic | kInitialAcyclic | kAccessible | kCoAccessible;
+  *props_ &= ~(kCyclic | kInitialCyclic | kNotAccessible | kNotCoAccessible);
+  fst_ = &fst;
+  start_ = fst.Start();
+  nstates_ = 0;
+  nscc_ = 0;
+  dfnumber_ = new vector<StateId>;
+  lowlink_ = new vector<StateId>;
+  onstack_ = new vector<bool>;
+  scc_stack_ = new vector<StateId>;
+}
+
+template <class A> inline
+bool SccVisitor<A>::InitState(StateId s, StateId root) {
+  scc_stack_->push_back(s);
+  while (dfnumber_->size() <= s) {
+    if (scc_)
+      scc_->push_back(-1);
+    if (access_)
+      access_->push_back(false);
+    coaccess_->push_back(false);
+    dfnumber_->push_back(-1);
+    lowlink_->push_back(-1);
+    onstack_->push_back(false);
+  }
+  (*dfnumber_)[s] = nstates_;
+  (*lowlink_)[s] = nstates_;
+  (*onstack_)[s] = true;
+  if (root == start_) {
+    if (access_)
+      (*access_)[s] = true;
+  } else {
+    if (access_)
+      (*access_)[s] = false;
+    *props_ |= kNotAccessible;
+    *props_ &= ~kAccessible;
+  }
+  ++nstates_;
+  return true;
+}
+
+template <class A> inline
+void SccVisitor<A>::FinishState(StateId s, StateId p, const A *) {
+  if (fst_->Final(s) != Weight::Zero())
+    (*coaccess_)[s] = true;
+  if ((*dfnumber_)[s] == (*lowlink_)[s]) {  // root of new SCC
+    bool scc_coaccess = false;
+    size_t i = scc_stack_->size();
+    StateId t;
+    do {
+      t = (*scc_stack_)[--i];
+      if ((*coaccess_)[t])
+        scc_coaccess = true;
+    } while (s != t);
+    do {
+      t = scc_stack_->back();
+      if (scc_)
+        (*scc_)[t] = nscc_;
+      if (scc_coaccess)
+        (*coaccess_)[t] = true;
+      (*onstack_)[t] = false;
+      scc_stack_->pop_back();
+    } while (s != t);
+    if (!scc_coaccess) {
+      *props_ |= kNotCoAccessible;
+      *props_ &= ~kCoAccessible;
+    }
+    ++nscc_;
+  }
+  if (p != kNoStateId) {
+    if ((*coaccess_)[s])
+      (*coaccess_)[p] = true;
+    if ((*lowlink_)[s] < (*lowlink_)[p])
+      (*lowlink_)[p] = (*lowlink_)[s];
+  }
+}
+
+
 // Trims an FST, removing states and arcs that are not on successful
 // paths. This version modifies its input.
 //
@@ -285,6 +314,46 @@ void Connect(MutableFst<Arc> *fst) {
   fst->SetProperties(kAccessible | kCoAccessible, kAccessible | kCoAccessible);
 }
 
+
+// Returns an acyclic FST where each SCC in the input FST has been
+// condensed to a single state with transitions between SCCs retained
+// and within SCCs dropped.  Also returns the mapping from an input
+// state 's' to an output state 'scc[s]'.
+template<class Arc>
+void Condense(const Fst<Arc> &ifst, MutableFst<Arc> *ofst,
+              vector<typename Arc::StateId> *scc) {
+  typedef typename Arc::StateId StateId;
+  typedef typename Arc::Weight Weight;
+
+  ofst->DeleteStates();
+  uint64 props = 0;
+  SccVisitor<Arc> scc_visitor(scc, 0, 0, &props);
+  DfsVisit(ifst, &scc_visitor);
+  for (StateId s = 0; s < scc->size(); ++s) {
+    StateId c = (*scc)[s];
+    while (c >= ofst->NumStates())
+      ofst->AddState();
+    if (s == ifst.Start())
+      ofst->SetStart(c);
+    Weight final = ifst.Final(s);
+    if (final != Weight::Zero())
+      ofst->SetFinal(c, Plus(ofst->Final(c), final));
+    for (ArcIterator< Fst<Arc> > aiter(ifst, s);
+         !aiter.Done();
+         aiter.Next()) {
+      Arc arc = aiter.Value();
+      StateId nextc = (*scc)[arc.nextstate];
+      if (nextc != c) {
+        while (nextc >= ofst->NumStates())
+          ofst->AddState();
+        arc.nextstate = nextc;
+        ofst->AddArc(c, arc);
+      }
+    }
+  }
+  ofst->SetProperties(kAcyclic | kInitialAcyclic, kAcyclic | kInitialAcyclic);
+}
+
 }  // namespace fst
 
 #endif  // FST_LIB_CONNECT_H__
