@@ -22,14 +22,17 @@
 #define FST_LIB_ENCODE_H__
 
 #include <climits>
-#include <tr1/unordered_map>
-using std::tr1::unordered_map;
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
 #include <string>
 #include <vector>
 using std::vector;
-#include <fst/map.h>
+
+#include <fst/arc-map.h>
 #include <fst/rmfinalepsilon.h>
 
+
 namespace fst {
 
 static const uint32 kEncodeLabels      = 0x0001;
@@ -154,78 +157,20 @@ template <class A>  class EncodeTable {
 
   // Given an encode arc Label decode back to input/output labels and costs
   const Tuple* Decode(Label key) const {
-    if (key < 1 || key > encode_tuples_.size())
-      LOG(FATAL) << "EncodeTable::Decode: unknown decode key: " << key;
+    if (key < 1 || key > encode_tuples_.size()) {
+      LOG(ERROR) << "EncodeTable::Decode: unknown decode key: " << key;
+      return 0;
+    }
     return encode_tuples_[key - 1];
   }
 
   size_t Size() const { return encode_tuples_.size(); }
 
-  bool Write(ostream &strm, const string &source) const {
-    WriteType(strm, kEncodeMagicNumber);
-    WriteType(strm, flags_);
-    int64 size = encode_tuples_.size();
-    WriteType(strm, size);
-    for (size_t i = 0;  i < size; ++i) {
-      const Tuple* tuple = encode_tuples_[i];
-      WriteType(strm, tuple->ilabel);
-      WriteType(strm, tuple->olabel);
-      tuple->weight.Write(strm);
-    }
+  bool Write(ostream &strm, const string &source) const;
 
-    if (flags_ & kEncodeHasISymbols)
-      isymbols_->Write(strm);
+  static EncodeTable<A> *Read(istream &strm, const string &source);
 
-    if (flags_ & kEncodeHasOSymbols)
-      osymbols_->Write(strm);
-
-    strm.flush();
-    if (!strm) {
-      LOG(ERROR) << "EncodeTable::Write: write failed: " << source;
-      return false;
-    }
-    return true;
-  }
-
-  bool Read(istream &strm, const string &source) {
-    encode_tuples_.clear();
-    encode_hash_.clear();
-    int32 magic_number = 0;
-    ReadType(strm, &magic_number);
-    if (magic_number != kEncodeMagicNumber) {
-      LOG(ERROR) << "EncodeTable::Read: Bad encode table header: " << source;
-      return false;
-    }
-    ReadType(strm, &flags_);
-    int64 size;
-    ReadType(strm, &size);
-    if (!strm) {
-      LOG(ERROR) << "EncodeTable::Read: read failed: " << source;
-      return false;
-    }
-    for (size_t i = 0; i < size; ++i) {
-      Tuple* tuple = new Tuple();
-      ReadType(strm, &tuple->ilabel);
-      ReadType(strm, &tuple->olabel);
-      tuple->weight.Read(strm);
-      if (!strm) {
-        LOG(ERROR) << "EncodeTable::Read: read failed: " << source;
-        return false;
-      }
-      encode_tuples_.push_back(tuple);
-      encode_hash_[encode_tuples_.back()] = encode_tuples_.size();
-    }
-
-    if (flags_ & kEncodeHasISymbols)
-      isymbols_ = SymbolTable::Read(strm, source);
-
-    if (flags_ & kEncodeHasOSymbols)
-      osymbols_ = SymbolTable::Read(strm, source);
-
-    return true;
-  }
-
-  const uint32 flags() const { return flags_ & kEncodeFlags; }
+  uint32 flags() const { return flags_ & kEncodeFlags; }
 
   int RefCount() const { return ref_count_.count(); }
   int IncrRefCount() { return ref_count_.Incr(); }
@@ -269,6 +214,75 @@ template <class A>  class EncodeTable {
   DISALLOW_COPY_AND_ASSIGN(EncodeTable);
 };
 
+template <class A> inline
+bool EncodeTable<A>::Write(ostream &strm, const string &source) const {
+  WriteType(strm, kEncodeMagicNumber);
+  WriteType(strm, flags_);
+  int64 size = encode_tuples_.size();
+  WriteType(strm, size);
+  for (size_t i = 0;  i < size; ++i) {
+    const Tuple* tuple = encode_tuples_[i];
+    WriteType(strm, tuple->ilabel);
+    WriteType(strm, tuple->olabel);
+    tuple->weight.Write(strm);
+  }
+
+  if (flags_ & kEncodeHasISymbols)
+    isymbols_->Write(strm);
+
+  if (flags_ & kEncodeHasOSymbols)
+    osymbols_->Write(strm);
+
+  strm.flush();
+  if (!strm) {
+    LOG(ERROR) << "EncodeTable::Write: write failed: " << source;
+    return false;
+  }
+  return true;
+}
+
+template <class A> inline
+EncodeTable<A> *EncodeTable<A>::Read(istream &strm, const string &source) {
+  int32 magic_number = 0;
+  ReadType(strm, &magic_number);
+  if (magic_number != kEncodeMagicNumber) {
+    LOG(ERROR) << "EncodeTable::Read: Bad encode table header: " << source;
+    return 0;
+  }
+  uint32 flags;
+  ReadType(strm, &flags);
+  EncodeTable<A> *table = new EncodeTable<A>(flags);
+
+  int64 size;
+  ReadType(strm, &size);
+  if (!strm) {
+    LOG(ERROR) << "EncodeTable::Read: read failed: " << source;
+    return 0;
+  }
+
+  for (size_t i = 0; i < size; ++i) {
+    Tuple* tuple = new Tuple();
+    ReadType(strm, &tuple->ilabel);
+    ReadType(strm, &tuple->olabel);
+    tuple->weight.Read(strm);
+    if (!strm) {
+      LOG(ERROR) << "EncodeTable::Read: read failed: " << source;
+      return 0;
+    }
+    table->encode_tuples_.push_back(tuple);
+    table->encode_hash_[table->encode_tuples_.back()] =
+        table->encode_tuples_.size();
+  }
+
+  if (flags & kEncodeHasISymbols)
+    table->isymbols_ = SymbolTable::Read(strm, source);
+
+  if (flags & kEncodeHasOSymbols)
+    table->osymbols_ = SymbolTable::Read(strm, source);
+
+  return table;
+}
+
 
 // A mapper to encode/decode weighted transducers. Encoding of an
 // Fst is useful for performing classical determinization or minimization
@@ -293,13 +307,16 @@ template <class A> class EncodeMapper {
   typedef typename A::Label  Label;
  public:
   EncodeMapper(uint32 flags, EncodeType type)
-    : flags_(flags), type_(type),
-      table_(new EncodeTable<A>(flags)) {}
+    : flags_(flags),
+      type_(type),
+      table_(new EncodeTable<A>(flags)),
+      error_(false) {}
 
   EncodeMapper(const EncodeMapper& mapper)
       : flags_(mapper.flags_),
         type_(mapper.type_),
-        table_(mapper.table_) {
+        table_(mapper.table_),
+        error_(false) {
     table_->IncrRefCount();
   }
 
@@ -307,7 +324,8 @@ template <class A> class EncodeMapper {
   EncodeMapper(const EncodeMapper& mapper, EncodeType type)
       : flags_(mapper.flags_),
         type_(type),
-        table_(mapper.table_) {
+        table_(mapper.table_),
+        error_(mapper.error_) {
     table_->IncrRefCount();
   }
 
@@ -315,41 +333,7 @@ template <class A> class EncodeMapper {
     if (!table_->DecrRefCount()) delete table_;
   }
 
-  A operator()(const A &arc) {
-    if (type_ == ENCODE) {  // labels and/or weights to single label
-      if ((arc.nextstate == kNoStateId && !(flags_ & kEncodeWeights)) ||
-          (arc.nextstate == kNoStateId && (flags_ & kEncodeWeights) &&
-           arc.weight == Weight::Zero())) {
-        return arc;
-      } else {
-        Label label = table_->Encode(arc);
-        return A(label,
-                 flags_ & kEncodeLabels ? label : arc.olabel,
-                 flags_ & kEncodeWeights ? Weight::One() : arc.weight,
-                 arc.nextstate);
-      }
-    } else if (type_ == DECODE) {
-      if (arc.nextstate == kNoStateId) {
-        return arc;
-      } else {
-        if (arc.ilabel == 0) return arc;
-        if (flags_ & kEncodeLabels && arc.ilabel != arc.olabel)
-          LOG(FATAL) << "EncodeMapper: Label-encoded arc has different "
-                     "input and output labels";
-        if (flags_ & kEncodeWeights && arc.weight != Weight::One())
-          LOG(FATAL) <<
-                     "EncodeMapper: Weight-encoded arc has non-trivial weight";
-        const typename EncodeTable<A>::Tuple* tuple =
-          table_->Decode(arc.ilabel);
-        return A(tuple->ilabel,
-                 flags_ & kEncodeLabels ? tuple->olabel : arc.olabel,
-                 flags_ & kEncodeWeights ? tuple->weight : arc.weight,
-                 arc.nextstate);
-      }
-    } else {
-      LOG(FATAL) << "EncodeMapper: Unknown operation";
-    }
-  }
+  A operator()(const A &arc);
 
   MapFinalAction FinalAction() const {
     return (type_ == ENCODE && (flags_ & kEncodeWeights)) ?
@@ -360,7 +344,10 @@ template <class A> class EncodeMapper {
 
   MapSymbolsAction OutputSymbolsAction() const { return MAP_CLEAR_SYMBOLS;}
 
-  uint64 Properties(uint64 props) {
+  uint64 Properties(uint64 inprops) {
+    uint64 outprops = inprops;
+    if (error_) outprops |= kError;
+
     uint64 mask = kFstProperties;
     if (flags_ & kEncodeLabels)
       mask &= kILabelInvariantProperties & kOLabelInvariantProperties;
@@ -368,11 +355,12 @@ template <class A> class EncodeMapper {
       mask &= kILabelInvariantProperties & kWeightInvariantProperties &
           (type_ == ENCODE ? kAddSuperFinalProperties :
            kRmSuperFinalProperties);
-    return props & mask;
+
+    return outprops & mask;
   }
 
-  const uint32 flags() const { return flags_; }
-  const EncodeType type() const { return type_; }
+  uint32 flags() const { return flags_; }
+  EncodeType type() const { return type_; }
   const EncodeTable<A> &table() const { return *table_; }
 
   bool Write(ostream &strm, const string& source) {
@@ -391,9 +379,8 @@ template <class A> class EncodeMapper {
   static EncodeMapper<A> *Read(istream &strm,
                                const string& source,
                                EncodeType type = ENCODE) {
-    EncodeTable<A> *table = new EncodeTable<A>(0);
-    bool r = table->Read(strm, source);
-    return r ? new EncodeMapper(table->flags(), type, table) : 0;
+    EncodeTable<A> *table = EncodeTable<A>::Read(strm, source);
+    return table ? new EncodeMapper(table->flags(), type, table) : 0;
   }
 
   static EncodeMapper<A> *Read(const string& filename,
@@ -401,7 +388,7 @@ template <class A> class EncodeMapper {
     ifstream strm(filename.c_str(), ifstream::in | ifstream::binary);
     if (!strm) {
       LOG(ERROR) << "EncodeMap: Can't open file: " << filename;
-      return false;
+      return NULL;
     }
     return Read(strm, filename, type);
   }
@@ -422,24 +409,69 @@ template <class A> class EncodeMapper {
   uint32 flags_;
   EncodeType type_;
   EncodeTable<A>* table_;
+  bool error_;
 
   explicit EncodeMapper(uint32 flags, EncodeType type, EncodeTable<A> *table)
-      : flags_(flags), type_(type), table_(table) {}
+      : flags_(flags), type_(type), table_(table), error_(false) {}
   void operator=(const EncodeMapper &);  // Disallow.
 };
 
+template <class A> inline
+A EncodeMapper<A>::operator()(const A &arc) {
+  if (type_ == ENCODE) {  // labels and/or weights to single label
+    if ((arc.nextstate == kNoStateId && !(flags_ & kEncodeWeights)) ||
+        (arc.nextstate == kNoStateId && (flags_ & kEncodeWeights) &&
+         arc.weight == Weight::Zero())) {
+      return arc;
+    } else {
+      Label label = table_->Encode(arc);
+      return A(label,
+               flags_ & kEncodeLabels ? label : arc.olabel,
+               flags_ & kEncodeWeights ? Weight::One() : arc.weight,
+               arc.nextstate);
+    }
+  } else {  // type_ == DECODE
+    if (arc.nextstate == kNoStateId) {
+      return arc;
+    } else {
+      if (arc.ilabel == 0) return arc;
+      if (flags_ & kEncodeLabels && arc.ilabel != arc.olabel) {
+        FSTERROR() << "EncodeMapper: Label-encoded arc has different "
+            "input and output labels";
+        error_ = true;
+      }
+      if (flags_ & kEncodeWeights && arc.weight != Weight::One()) {
+        FSTERROR() <<
+            "EncodeMapper: Weight-encoded arc has non-trivial weight";
+        error_ = true;
+      }
+      const typename EncodeTable<A>::Tuple* tuple = table_->Decode(arc.ilabel);
+      if (!tuple) {
+        FSTERROR() << "EncodeMapper: decode failed";
+        error_ = true;
+        return A(kNoLabel, kNoLabel, Weight::NoWeight(), arc.nextstate);
+      } else {
+        return A(tuple->ilabel,
+                 flags_ & kEncodeLabels ? tuple->olabel : arc.olabel,
+                 flags_ & kEncodeWeights ? tuple->weight : arc.weight,
+                 arc.nextstate);
+      }
+    }
+  }
+}
+
 
 // Complexity: O(nstates + narcs)
 template<class A> inline
 void Encode(MutableFst<A> *fst, EncodeMapper<A>* mapper) {
   mapper->SetInputSymbols(fst->InputSymbols());
   mapper->SetOutputSymbols(fst->OutputSymbols());
-  Map(fst, mapper);
+  ArcMap(fst, mapper);
 }
 
 template<class A> inline
 void Decode(MutableFst<A>* fst, const EncodeMapper<A>& mapper) {
-  Map(fst, EncodeMapper<A>(mapper, DECODE));
+  ArcMap(fst, EncodeMapper<A>(mapper, DECODE));
   RmFinalEpsilon(fst);
   fst->SetInputSymbols(mapper.InputSymbols());
   fst->SetOutputSymbols(mapper.OutputSymbols());
@@ -453,28 +485,32 @@ void Decode(MutableFst<A>* fst, const EncodeMapper<A>& mapper) {
 // - Traversal: O(nstates_visited + narcs_visited), assuming constant
 //   time to visit an input state or arc.
 template <class A>
-class EncodeFst : public MapFst<A, A, EncodeMapper<A> > {
+class EncodeFst : public ArcMapFst<A, A, EncodeMapper<A> > {
  public:
   typedef A Arc;
   typedef EncodeMapper<A> C;
+  typedef ArcMapFstImpl< A, A, EncodeMapper<A> > Impl;
+  using ImplToFst<Impl>::GetImpl;
 
   EncodeFst(const Fst<A> &fst, EncodeMapper<A>* encoder)
-      : MapFst<A, A, C>(fst, encoder, MapFstOptions()) {
+      : ArcMapFst<A, A, C>(fst, encoder, ArcMapFstOptions()) {
     encoder->SetInputSymbols(fst.InputSymbols());
     encoder->SetOutputSymbols(fst.OutputSymbols());
   }
 
   EncodeFst(const Fst<A> &fst, const EncodeMapper<A>& encoder)
-      : MapFst<A, A, C>(fst, encoder, MapFstOptions()) {}
+      : ArcMapFst<A, A, C>(fst, encoder, ArcMapFstOptions()) {}
 
   // See Fst<>::Copy() for doc.
   EncodeFst(const EncodeFst<A> &fst, bool copy = false)
-      : MapFst<A, A, C>(fst, copy) {}
+      : ArcMapFst<A, A, C>(fst, copy) {}
 
   // Get a copy of this EncodeFst. See Fst<>::Copy() for further doc.
   virtual EncodeFst<A> *Copy(bool safe = false) const {
-    if (safe)
-      LOG(FATAL) << "EncodeFst::Copy(true): not allowed.";
+    if (safe) {
+      FSTERROR() << "EncodeFst::Copy(true): not allowed.";
+      GetImpl()->SetProperties(kError, kError);
+    }
     return new EncodeFst(*this);
   }
 };
@@ -487,24 +523,24 @@ class EncodeFst : public MapFst<A, A, EncodeMapper<A> > {
 // - Traversal: O(nstates_visited + narcs_visited), assuming constant
 //   time to visit an input state or arc.
 template <class A>
-class DecodeFst : public MapFst<A, A, EncodeMapper<A> > {
+class DecodeFst : public ArcMapFst<A, A, EncodeMapper<A> > {
  public:
   typedef A Arc;
   typedef EncodeMapper<A> C;
-  typedef MapFstImpl< A, A, EncodeMapper<A> > Impl;
+  typedef ArcMapFstImpl< A, A, EncodeMapper<A> > Impl;
   using ImplToFst<Impl>::GetImpl;
 
   DecodeFst(const Fst<A> &fst, const EncodeMapper<A>& encoder)
-      : MapFst<A, A, C>(fst,
+      : ArcMapFst<A, A, C>(fst,
                             EncodeMapper<A>(encoder, DECODE),
-                            MapFstOptions()) {
+                            ArcMapFstOptions()) {
     GetImpl()->SetInputSymbols(encoder.InputSymbols());
     GetImpl()->SetOutputSymbols(encoder.OutputSymbols());
   }
 
   // See Fst<>::Copy() for doc.
   DecodeFst(const DecodeFst<A> &fst, bool safe = false)
-      : MapFst<A, A, C>(fst, safe) {}
+      : ArcMapFst<A, A, C>(fst, safe) {}
 
   // Get a copy of this DecodeFst. See Fst<>::Copy() for further doc.
   virtual DecodeFst<A> *Copy(bool safe = false) const {
@@ -516,40 +552,40 @@ class DecodeFst : public MapFst<A, A, EncodeMapper<A> > {
 // Specialization for EncodeFst.
 template <class A>
 class StateIterator< EncodeFst<A> >
-    : public StateIterator< MapFst<A, A, EncodeMapper<A> > > {
+    : public StateIterator< ArcMapFst<A, A, EncodeMapper<A> > > {
  public:
   explicit StateIterator(const EncodeFst<A> &fst)
-      : StateIterator< MapFst<A, A, EncodeMapper<A> > >(fst) {}
+      : StateIterator< ArcMapFst<A, A, EncodeMapper<A> > >(fst) {}
 };
 
 
 // Specialization for EncodeFst.
 template <class A>
 class ArcIterator< EncodeFst<A> >
-    : public ArcIterator< MapFst<A, A, EncodeMapper<A> > > {
+    : public ArcIterator< ArcMapFst<A, A, EncodeMapper<A> > > {
  public:
   ArcIterator(const EncodeFst<A> &fst, typename A::StateId s)
-      : ArcIterator< MapFst<A, A, EncodeMapper<A> > >(fst, s) {}
+      : ArcIterator< ArcMapFst<A, A, EncodeMapper<A> > >(fst, s) {}
 };
 
 
 // Specialization for DecodeFst.
 template <class A>
 class StateIterator< DecodeFst<A> >
-    : public StateIterator< MapFst<A, A, EncodeMapper<A> > > {
+    : public StateIterator< ArcMapFst<A, A, EncodeMapper<A> > > {
  public:
   explicit StateIterator(const DecodeFst<A> &fst)
-      : StateIterator< MapFst<A, A, EncodeMapper<A> > >(fst) {}
+      : StateIterator< ArcMapFst<A, A, EncodeMapper<A> > >(fst) {}
 };
 
 
 // Specialization for DecodeFst.
 template <class A>
 class ArcIterator< DecodeFst<A> >
-    : public ArcIterator< MapFst<A, A, EncodeMapper<A> > > {
+    : public ArcIterator< ArcMapFst<A, A, EncodeMapper<A> > > {
  public:
   ArcIterator(const DecodeFst<A> &fst, typename A::StateId s)
-      : ArcIterator< MapFst<A, A, EncodeMapper<A> > >(fst, s) {}
+      : ArcIterator< ArcMapFst<A, A, EncodeMapper<A> > >(fst, s) {}
 };
 
 
@@ -558,6 +594,6 @@ typedef EncodeFst<StdArc> StdEncodeFst;
 
 typedef DecodeFst<StdArc> StdDecodeFst;
 
-}
+}  // namespace fst
 
 #endif  // FST_LIB_ENCODE_H__
