@@ -30,11 +30,16 @@ using std::vector;
 namespace fst {
 
 template <class Arc>
-void CountStatesAndArcs(const Fst<Arc> &fst, size_t *nstate, size_t *narc) {
+void CountStatesAndArcs(const Fst<Arc> &fst,
+                        size_t *nstate,
+                        size_t *narc,
+                        size_t *nfinal) {
   StateIterator<Fst<Arc> > siter(fst);
   for (; !siter.Done(); siter.Next(), ++(*nstate)) {
     ArcIterator<Fst<Arc> > aiter(fst, siter.Value());
     for (; !aiter.Done(); aiter.Next(), ++(*narc)) {}
+    if (fst.Final(siter.Value()) != Arc::Weight::Zero())
+      ++(*nfinal);
   }
 }
 
@@ -43,9 +48,10 @@ struct KeyInfo {
   string type;
   size_t nstate;
   size_t narc;
+  size_t nfinal;
 
-  KeyInfo(string k, string t, int64 ns = 0, int64 na = 0)
-  : key(k), type(t), nstate(ns), narc(na) {}
+  KeyInfo(string k, string t, int64 ns = 0, int64 na = 0, int nf = 0)
+      : key(k), type(t), nstate(ns), narc(na), nfinal(nf) {}
 };
 
 template <class Arc>
@@ -58,7 +64,7 @@ void FarInfo(const vector<string> &filenames, const string &begin_key,
     far_reader->Find(begin_key);
 
   vector<KeyInfo> *infos = list_fsts ? new vector<KeyInfo>() : 0;
-  size_t nfst = 0, nstate = 0, narc = 0;
+  size_t nfst = 0, nstate = 0, narc = 0, nfinal = 0;
   set<string> fst_types;
   for (; !far_reader->Done(); far_reader->Next()) {
     string key = far_reader->GetKey();
@@ -69,12 +75,13 @@ void FarInfo(const vector<string> &filenames, const string &begin_key,
     fst_types.insert(fst.Type());
     if (infos) {
       KeyInfo info(key, fst.Type());
-      CountStatesAndArcs(fst, &info.nstate, &info.narc);
+      CountStatesAndArcs(fst, &info.nstate, &info.narc, &info.nfinal);
       nstate += info.nstate;
-      nstate += info.narc;
+      narc += info.narc;
+      nfinal += info.nfinal;
       infos->push_back(info);
     } else {
-      CountStatesAndArcs(fst, &nstate, &narc);
+      CountStatesAndArcs(fst, &nstate, &narc, &nfinal);
     }
   }
 
@@ -94,9 +101,11 @@ void FarInfo(const vector<string> &filenames, const string &begin_key,
     cout << std::left << setw(50) << "# of FSTs" << nfst << endl;
     cout << std::left << setw(50) << "total # of states" << nstate << endl;
     cout << std::left << setw(50) << "total # of arcs" << narc << endl;
+    cout << std::left << setw(50) << "total # of final states" << nfinal
+         << endl;
 
   } else  {
-    int wkey = 10, wtype = 10, wnstate = 16, wnarc = 16;
+    int wkey = 10, wtype = 10, wnstate = 14, wnarc = 12, wnfinal = 20;
     for (size_t i = 0; i < infos->size(); ++i) {
       const KeyInfo &info = (*infos)[i];
       if (info.key.size() + 2 > wkey)
@@ -107,17 +116,19 @@ void FarInfo(const vector<string> &filenames, const string &begin_key,
         wnstate = ceil(log10(info.nstate)) + 2;
       if (ceil(log10(info.narc)) + 2 > wnarc)
         wnarc = ceil(log10(info.narc)) + 2;
+      if (ceil(log10(info.nfinal)) + 2 > wnfinal)
+        wnfinal = ceil(log10(info.nfinal)) + 2;
     }
 
     cout << std::left << setw(wkey) << "key" << setw(wtype) << "type"
-         << std::right << setw(wnstate) << "# of states"
-         << setw(wnarc) << "# of arcs" << endl;
+         << std::right << setw(wnstate) << "# of states" << setw(wnarc)
+         << "# of arcs" << setw(wnfinal) << "# of final states" << endl;
 
     for (size_t i = 0; i < infos->size(); ++i) {
       const KeyInfo &info = (*infos)[i];
       cout << std::left << setw(wkey) << info.key << setw(wtype) << info.type
            << std::right << setw(wnstate) << info.nstate
-           << setw(wnarc) << info.narc << endl;
+           << setw(wnarc) << info.narc  << setw(wnfinal) << info.nfinal << endl;
     }
   }
 }
