@@ -122,27 +122,27 @@ class StringFactor {
 
 
 // Factor a GallicWeight using StringFactor.
-template <class L, class W, StringType S = STRING_LEFT>
+template <class L, class W, GallicType G = GALLIC_LEFT>
 class GallicFactor {
  public:
-  GallicFactor(const GallicWeight<L, W, S> &w)
+  GallicFactor(const GallicWeight<L, W, G> &w)
       : weight_(w), done_(w.Value1().Size() <= 1) {}
 
   bool Done() const { return done_; }
 
   void Next() { done_ = true; }
 
-  pair< GallicWeight<L, W, S>, GallicWeight<L, W, S> > Value() const {
-    StringFactor<L, S> iter(weight_.Value1());
-    GallicWeight<L, W, S> w1(iter.Value().first, weight_.Value2());
-    GallicWeight<L, W, S> w2(iter.Value().second, W::One());
+  pair< GallicWeight<L, W, G>, GallicWeight<L, W, G> > Value() const {
+    StringFactor<L, GALLIC_STRING_TYPE(G)> iter(weight_.Value1());
+    GallicWeight<L, W, G> w1(iter.Value().first, weight_.Value2());
+    GallicWeight<L, W, G> w2(iter.Value().second, W::One());
     return make_pair(w1, w2);
   }
 
   void Reset() { done_ = weight_.Value1().Size() <= 1; }
 
  private:
-  GallicWeight<L, W, S> weight_;
+  GallicWeight<L, W, G> weight_;
   bool done_;
 };
 
