@@ -32,6 +32,7 @@
 #include <fst/types.h>
 
 #include <fst/arc.h>
+#include <fst/memory.h>
 #include <fst/properties.h>
 #include <fst/register.h>
 #include <iostream>
@@ -53,6 +54,11 @@ template <class A> class ArcIteratorData;
 template <class A> class MatcherBase;
 
 struct FstReadOptions {
+  // FileReadMode(s) are advisory, there are many conditions than prevent a
+  // file from being mapped, READ mode will be selected in these cases with
+  // a warning indicating why it was chosen.
+  enum FileReadMode { READ, MAP };
+
   string source;                // Where you're reading from
   const FstHeader *header;      // Pointer to Fst header. If non-zero, use
                                 // this info (don't read a stream header)
@@ -60,19 +66,22 @@ struct FstReadOptions {
                                 // this info (read and skip stream isymbols)
   const SymbolTable* osymbols;  // Pointer to output symbols. If non-zero, use
                                 // this info (read and skip stream osymbols)
+  FileReadMode mode;            // Read or map files (advisory, if possible)
+  bool read_isymbols;           // Read isymbols, if any, default true
+  bool read_osymbols;           // Read osymbols, if any, default true
 
-  explicit FstReadOptions(const string& src = "<unspecfied>",
+  explicit FstReadOptions(const string& src = "<unspecified>",
                           const FstHeader *hdr = 0,
                           const SymbolTable* isym = 0,
-                          const SymbolTable* osym = 0)
-      : source(src), header(hdr), isymbols(isym), osymbols(osym) {}
+                          const SymbolTable* osym = 0);
 
   explicit FstReadOptions(const string& src,
                           const SymbolTable* isym,
-                          const SymbolTable* osym = 0)
-      : source(src), header(0), isymbols(isym), osymbols(osym) {}
-};
+                          const SymbolTable* osym = 0);
 
+  // Helper function to convert strings FileReadModes into their enum value.
+  static FileReadMode ReadMode(const string &mode);
+};
 
 struct FstWriteOptions {
   string source;                 // Where you're writing to
@@ -217,7 +226,7 @@ class Fst {
       return 0;
     }
     return reader(strm, ropts);
-  };
+  }
 
   // Read an Fst from a file; return NULL on error
   // Empty filename reads from standard input
@@ -431,7 +440,6 @@ template <class A> struct ArcIteratorData {
   int *ref_count;            // ... and reference count if non-zero
 };
 
-
 // Generic arc iterator, templated on the FST definition
 // - a wrapper around pointer to specific one.
 // Here is a typical use: \code
@@ -515,6 +523,29 @@ class ArcIterator {
   DISALLOW_COPY_AND_ASSIGN(ArcIterator);
 };
 
+}  // namespace fst
+
+// ArcIterator placement operator new and destroy function.
+// new needs to be in the global namespace
+
+template <class F>
+void* operator new(size_t size,
+                   fst::MemoryPool< fst::ArcIterator<F> > *pool) {
+  return pool->Allocate();
+}
+
+namespace fst {
+
+template <class F>
+inline void Destroy(fst::ArcIterator<F> *aiter,
+                    fst::MemoryPool< fst::ArcIterator<F> > *pool) {
+  if (aiter) {
+    aiter->~ArcIterator<F>();
+    pool->Free(aiter);
+  }
+}
+
+
 //
 // MATCHER DEFINITIONS
 //
@@ -706,12 +737,13 @@ template <class A> class FstImpl {
   // This method is used in the cross-type serialization methods Fst::WriteFst.
   static void WriteFstHeader(const Fst<A> &fst, ostream &strm,
                              const FstWriteOptions& opts, int version,
-                             const string &type, FstHeader *hdr) {
+                             const string &type, uint64 properties,
+                             FstHeader *hdr) {
     if (opts.write_header) {
       hdr->SetFstType(type);
       hdr->SetArcType(A::Type());
       hdr->SetVersion(version);
-      hdr->SetProperties(fst.Properties(kFstProperties, false));
+      hdr->SetProperties(properties);
       int32 file_flags = 0;
       if (fst.InputSymbols() && opts.write_isymbols)
         file_flags |= FstHeader::HAS_ISYMBOLS;
@@ -737,14 +769,14 @@ template <class A> class FstImpl {
   // returns true on success, false on failure.
   static bool UpdateFstHeader(const Fst<A> &fst, ostream &strm,
                               const FstWriteOptions& opts, int version,
-                              const string &type, FstHeader *hdr,
-                              size_t header_offset) {
+                              const string &type, uint64 properties,
+                              FstHeader *hdr, size_t header_offset) {
     strm.seekp(header_offset);
     if (!strm) {
       LOG(ERROR) << "Fst::UpdateFstHeader: write failed: " << opts.source;
       return false;
     }
-    WriteFstHeader(fst, strm, opts, version, type, hdr);
+    WriteFstHeader(fst, strm, opts, version, type, properties, hdr);
     if (!strm) {
       LOG(ERROR) << "Fst::UpdateFstHeader: write failed: " << opts.source;
       return false;
@@ -803,9 +835,14 @@ bool FstImpl<A>::ReadHeader(istream &strm, const FstReadOptions& opts,
   properties_ = hdr->Properties();
   if (hdr->GetFlags() & FstHeader::HAS_ISYMBOLS)
     isymbols_ = SymbolTable::Read(strm, opts.source);
+  // Input symbol table not wanted; delete
+  if (!opts.read_isymbols)
+    SetInputSymbols(0);
   if (hdr->GetFlags() & FstHeader::HAS_OSYMBOLS)
-    osymbols_ =SymbolTable::Read(strm, opts.source);
-
+    osymbols_ = SymbolTable::Read(strm, opts.source);
+  // Output symbol table not wanted; delete
+  if (!opts.read_osymbols)
+    SetOutputSymbols(0);
   if (opts.isymbols) {
     delete isymbols_;
     isymbols_ = opts.isymbols->Copy();
@@ -931,6 +968,14 @@ void FstToString(const Fst<A> &fst, string *result) {
   *result = ostrm.str();
 }
 
+template <class A>
+void FstToString(const Fst<A> &fst, string *result,
+                 const FstWriteOptions& options) {
+  ostringstream ostrm;
+  fst.Write(ostrm, options);
+  *result = ostrm.str();
+}
+
 template <class A>
 Fst<A> *StringToFst(const string &s) {
   istringstream istrm(s);
@@ -939,4 +984,6 @@ Fst<A> *StringToFst(const string &s) {
 
 }  // namespace fst
 
+
+
 #endif  // FST_LIB_FST_H__
