@@ -25,9 +25,11 @@
 using std::vector;
 #include <algorithm>
 
+
 #include <fst/queue.h>
 
 
+
 namespace fst {
 
 template <typename T> class PartitionIterator;
@@ -215,7 +217,7 @@ class Partition {
     return class_size_;
   }
 
-  const size_t class_size(T class_id)  const {
+  size_t class_size(T class_id) const {
     return class_size_[class_id];
   }
 
@@ -283,6 +285,6 @@ class PartitionIterator {
 
   T class_id_;
 };
-}
+}  // namespace fst
 
 #endif  // FST_LIB_PARTITION_H__
