@@ -23,9 +23,11 @@
 
 #include <stack>
 #include <string>
+
 #include <fst/pair-weight.h>
 #include <fst/weight.h>
 
+
 namespace fst {
 
 // Product semiring: W1 * W2
@@ -34,6 +36,7 @@ class ProductWeight : public PairWeight<W1, W2> {
  public:
   using PairWeight<W1, W2>::Zero;
   using PairWeight<W1, W2>::One;
+  using PairWeight<W1, W2>::NoWeight;
   using PairWeight<W1, W2>::Quantize;
   using PairWeight<W1, W2>::Reverse;
 
@@ -56,6 +59,12 @@ class ProductWeight : public PairWeight<W1, W2> {
     return one;
   }
 
+  static const ProductWeight<W1, W2> &NoWeight() {
+    static const ProductWeight<W1, W2> no_weight(
+        PairWeight<W1, W2>::NoWeight());
+    return no_weight;
+  }
+
   static const string &Type() {
     static const string type = W1::Type() + "_X_" + W2::Type();
     return type;
@@ -75,8 +84,6 @@ class ProductWeight : public PairWeight<W1, W2> {
   ReverseWeight Reverse() const {
     return PairWeight<W1, W2>::Reverse();
   }
-
-
 };
 
 template <class W1, class W2>
@@ -101,6 +108,6 @@ inline ProductWeight<W1, W2> Divide(const ProductWeight<W1, W2> &w,
                                Divide(w.Value2(), v.Value2(), typ));
 }
 
-}  // namespace fst;
+}  // namespace fst
 
 #endif  // FST_LIB_PRODUCT_WEIGHT_H__
