@@ -21,14 +21,16 @@
 #ifndef FST_LIB_RMFINALEPSILON_H__
 #define FST_LIB_RMFINALEPSILON_H__
 
-#include <tr1/unordered_set>
-using std::tr1::unordered_set;
+#include <unordered_set>
+using std::unordered_set;
+using std::unordered_multiset;
 #include <vector>
 using std::vector;
 
 #include <fst/connect.h>
 #include <fst/mutable-fst.h>
 
+
 namespace fst {
 
 template<class A>
@@ -100,6 +102,6 @@ void RmFinalEpsilon(MutableFst<A>* fst) {
   Connect(fst);
 }
 
-}
+}  // namespace fst
 
 #endif  // FST_LIB_RMFINALEPSILON_H__
