@@ -23,9 +23,9 @@
 #define FST_LIB_STATE_MAP_H__
 
 #include <algorithm>
-#include <tr1/unordered_map>
-using std::tr1::unordered_map;
-using std::tr1::unordered_multimap;
+#include <unordered_map>
+using std::unordered_map;
+using std::unordered_multimap;
 #include <string>
 #include <utility>
 using std::pair; using std::make_pair;
@@ -191,8 +191,6 @@ class StateMapFstImpl : public CacheImpl<B> {
   using FstImpl<B>::SetInputSymbols;
   using FstImpl<B>::SetOutputSymbols;
 
-  using VectorFstBaseImpl<typename CacheImpl<B>::State>::NumStates;
-
   using CacheImpl<B>::PushArc;
   using CacheImpl<B>::HasArcs;
   using CacheImpl<B>::HasFinal;
@@ -335,7 +333,8 @@ class StateMapFst : public ImplToFst< StateMapFstImpl<A, B, C> > {
   typedef B Arc;
   typedef typename B::Weight Weight;
   typedef typename B::StateId StateId;
-  typedef CacheState<B> State;
+  typedef DefaultCacheStore<B> Store;
+  typedef typename Store::State State;
   typedef StateMapFstImpl<A, B, C> Impl;
 
   StateMapFst(const Fst<A> &fst, const C &mapper,
@@ -495,7 +494,7 @@ class ArcSumMapper {
 
  private:
   struct Compare {
-    bool operator()(const A& x, const A& y) {
+    bool operator()(const A &x, const A &y) const {
       if (x.ilabel < y.ilabel) return true;
       if (x.ilabel > y.ilabel) return false;
       if (x.olabel < y.olabel) return true;
@@ -570,7 +569,7 @@ class ArcUniqueMapper {
 
  private:
   struct Compare {
-    bool operator()(const A& x, const A& y) {
+    bool operator()(const A &x, const A &y) const {
       if (x.ilabel < y.ilabel) return true;
       if (x.ilabel > y.ilabel) return false;
       if (x.olabel < y.olabel) return true;
@@ -582,7 +581,7 @@ class ArcUniqueMapper {
   };
 
   struct Equal {
-    bool operator()(const A& x, const A& y) {
+    bool operator()(const A &x, const A &y) const {
       return (x.ilabel == y.ilabel &&
               x.olabel == y.olabel &&
               x.nextstate == y.nextstate &&
