@@ -24,6 +24,7 @@
 #include <fst/fst.h>
 #include <fst/test-properties.h>
 
+
 namespace fst {
 
 // Verifies that an Fst's contents are sane.
@@ -81,7 +82,7 @@ bool Verify(const Fst<Arc> &fst, bool allow_negative_labels = false) {
                    << " is missing from output symbol table \""
                    << osyms->Name() << "\"";
         return false;
-      } else if (!arc.weight.Member() || arc.weight == Weight::Zero()) {
+      } else if (!arc.weight.Member()) {
         LOG(ERROR) << "Verify: Fst weight of arc at position "
                    << na << " of state " << s << " is invalid";
         return false;
@@ -103,6 +104,11 @@ bool Verify(const Fst<Arc> &fst, bool allow_negative_labels = false) {
     }
   }
   uint64 fst_props = fst.Properties(kFstProperties, false);
+  if (fst_props & kError) {
+    LOG(ERROR) << "Verify: Fst error property is set";
+    return false;
+  }
+
   uint64 known_props;
   uint64 test_props = ComputeProperties(fst, kFstProperties, &known_props,
                                         false);
