@@ -18,6 +18,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <unistd.h>
 
 namespace fst {
 
@@ -37,15 +38,24 @@ MappedFile::~MappedFile() {
         LOG(ERROR) << "failed to unmap region: "<< strerror(errno);
       }
     } else {
-      operator delete(region_.data);
+      if (region_.data != 0) {
+        operator delete(static_cast<char*>(region_.data) - region_.offset);
+      }
     }
   }
 }
 
-MappedFile* MappedFile::Allocate(size_t size) {
+MappedFile* MappedFile::Allocate(size_t size, int align) {
   MemoryRegion region;
-  region.data = size == 0 ? NULL : operator new(size);
-  region.mmap = NULL;
+  region.data = 0;
+  region.offset = 0;
+  if (size > 0) {
+    char *buffer = static_cast<char*>(operator new(size + align));
+    size_t address = reinterpret_cast<size_t>(buffer);
+    region.offset = kArchAlignment - (address % align);
+    region.data = buffer + region.offset;
+  }
+  region.mmap = 0;
   region.size = size;
   return new MappedFile(region);
 }
@@ -55,17 +65,18 @@ MappedFile* MappedFile::Borrow(void *data) {
   region.data = data;
   region.mmap = data;
   region.size = 0;
+  region.offset = 0;
   return new MappedFile(region);
 }
 
-MappedFile* MappedFile::Map(istream* s, const FstReadOptions &opts,
-                            size_t size) {
-  size_t pos = s->tellg();
-  if (opts.mode == FstReadOptions::MAP && pos >= 0 &&
-      pos % kArchAlignment == 0) {
-    int fd = open(opts.source.c_str(), O_RDONLY);
+MappedFile* MappedFile::Map(istream* s, bool memorymap,
+                            const string& source, size_t size) {
+  std::streampos spos = s->tellg();
+  if (memorymap && spos >= 0 && spos % kArchAlignment == 0) {
+    size_t pos = spos;
+    int fd = open(source.c_str(), O_RDONLY);
     if (fd != -1) {
-      int pagesize = getpagesize();
+      int pagesize = sysconf(_SC_PAGESIZE);
       off_t offset = pos % pagesize;
       off_t upsize = size + offset;
       void *map = mmap(0, upsize, PROT_READ, MAP_SHARED, fd, pos - offset);
@@ -75,11 +86,12 @@ MappedFile* MappedFile::Map(istream* s, const FstReadOptions &opts,
         region.mmap = map;
         region.size = upsize;
         region.data = reinterpret_cast<void*>(data + offset);
+        region.offset = offset;
         MappedFile *mmf = new MappedFile(region);
         s->seekg(pos + size, ios::beg);
         if (s) {
           VLOG(1) << "mmap'ed region of " << size << " at offset " << pos
-                  << " from " << opts.source.c_str() << " to addr " << map;
+                  << " from " << source << " to addr " << map;
           return mmf;
         }
         delete mmf;
@@ -89,9 +101,9 @@ MappedFile* MappedFile::Map(istream* s, const FstReadOptions &opts,
     }
   }
   // If all else fails resort to reading from file into allocated buffer.
-  if (opts.mode != FstReadOptions::READ) {
-    LOG(WARNING) << "File mapping at offset " << pos << " of file "
-                 << opts.source << " could not be honored, reading instead.";
+  if (memorymap) {
+    LOG(WARNING) << "File mapping at offset " << spos << " of file "
+                 << source << " could not be honored, reading instead.";
   }
   MappedFile* mf = Allocate(size);
   if (!s->read(reinterpret_cast<char*>(mf->mutable_data()), size)) {
