@@ -19,12 +19,12 @@
 // Functions for updating property bits for various FST operations and
 // string names of the properties.
 
+#include <fst/properties.h>
+
+#include <stddef.h>
 #include <vector>
 using std::vector;
 
-#include <fst/properties.h>
-#include <fst/encode.h>
-
 namespace fst {
 
 // These functions determine the properties associated with the FST
@@ -36,7 +36,7 @@ namespace fst {
 
 // Properties for a concatenatively-closed FST.
 uint64 ClosureProperties(uint64 inprops, bool star, bool delayed) {
-  uint64 outprops = (kAcceptor | kUnweighted | kAccessible) & inprops;
+  uint64 outprops = (kError | kAcceptor | kUnweighted | kAccessible) & inprops;
   if (!delayed)
        outprops |= (kExpanded | kMutable | kCoAccessible |
                     kNotTopSorted | kNotString) & inprops;
@@ -52,7 +52,8 @@ uint64 ComplementProperties(uint64 inprops) {
   uint64 outprops = kAcceptor | kUnweighted | kNoEpsilons |
                     kNoIEpsilons | kNoOEpsilons |
                     kIDeterministic | kODeterministic | kAccessible;
-  outprops |= (kILabelSorted | kOLabelSorted | kInitialCyclic) & inprops;
+  outprops |= (kError | kILabelSorted | kOLabelSorted | kInitialCyclic) &
+      inprops;
   if (inprops & kAccessible)
     outprops |= kNotILabelSorted | kNotOLabelSorted | kCyclic;
   return outprops;
@@ -60,15 +61,15 @@ uint64 ComplementProperties(uint64 inprops) {
 
 // Properties for a composed FST.
 uint64 ComposeProperties(uint64 inprops1, uint64 inprops2) {
-  uint64 outprops;
+  uint64 outprops = kError & (inprops1 | inprops2);
   if (inprops1 & kAcceptor && inprops2 & kAcceptor) {
-    outprops = kAcceptor | kAccessible;
+    outprops |= kAcceptor | kAccessible;
     outprops |= (kNoEpsilons | kNoIEpsilons | kNoOEpsilons | kAcyclic |
                  kInitialAcyclic) & inprops1 & inprops2;
     if (kNoIEpsilons & inprops1 & inprops2)
       outprops |= (kIDeterministic | kODeterministic) & inprops1 & inprops2;
   } else {
-    outprops = kAccessible;
+    outprops |= kAccessible;
     outprops |= (kAcceptor | kNoIEpsilons | kAcyclic | kInitialAcyclic) &
         inprops1 & inprops2;
     if (kNoIEpsilons & inprops1 & inprops2)
@@ -81,6 +82,7 @@ uint64 ComposeProperties(uint64 inprops1, uint64 inprops2) {
 uint64 ConcatProperties(uint64 inprops1, uint64 inprops2, bool delayed) {
   uint64 outprops =
     (kAcceptor | kUnweighted | kAcyclic) & inprops1 & inprops2;
+  outprops |= kError & (inprops1 | inprops2);
 
   bool empty1 = delayed;  // Can fst1 be the empty machine?
   bool empty2 = delayed;  // Can fst2 be the empty machine?
@@ -115,8 +117,10 @@ uint64 DeterminizeProperties(uint64 inprops, bool has_subsequential_label) {
   uint64 outprops = kAccessible;
   if (((kAcceptor | kNoIEpsilons) & inprops) || has_subsequential_label)
     outprops |= kIDeterministic;
-  outprops |= (kAcceptor | kNoEpsilons | kAcyclic |
+  outprops |= (kError | kAcceptor | kAcyclic |
                kInitialAcyclic | kCoAccessible | kString) & inprops;
+  if (inprops & kNoIEpsilons)
+    outprops |= kNoEpsilons & inprops;
   if (inprops & kAccessible)
      outprops |= (kNotAcceptor | kEpsilons | kIEpsilons | kOEpsilons |
                   kCyclic) & inprops;
@@ -129,7 +133,7 @@ uint64 DeterminizeProperties(uint64 inprops, bool has_subsequential_label) {
 
 // Properties for factored weight FST.
 uint64 FactorWeightProperties(uint64 inprops) {
-  uint64 outprops = (kExpanded | kMutable | kAcceptor |
+  uint64 outprops = (kExpanded | kMutable | kError | kAcceptor |
                      kAcyclic | kAccessible | kCoAccessible) & inprops;
   if (inprops & kAccessible)
     outprops |= (kNotAcceptor | kNonIDeterministic | kNonODeterministic |
@@ -141,7 +145,7 @@ uint64 FactorWeightProperties(uint64 inprops) {
 
 // Properties for an inverted FST.
 uint64 InvertProperties(uint64 inprops) {
-  uint64 outprops = (kExpanded | kMutable | kAcceptor | kNotAcceptor |
+  uint64 outprops = (kExpanded | kMutable | kError | kAcceptor | kNotAcceptor |
                      kEpsilons | kNoEpsilons | kWeighted | kUnweighted |
                      kCyclic | kAcyclic | kInitialCyclic | kInitialAcyclic |
                      kTopSorted | kNotTopSorted |
@@ -180,7 +184,7 @@ uint64 InvertProperties(uint64 inprops) {
 // Properties for a projected FST.
 uint64 ProjectProperties(uint64 inprops, bool project_input) {
   uint64 outprops = kAcceptor;
-  outprops |= (kExpanded | kMutable | kWeighted | kUnweighted |
+  outprops |= (kExpanded | kMutable | kError | kWeighted | kUnweighted |
                kCyclic | kAcyclic | kInitialCyclic | kInitialAcyclic |
                kTopSorted | kNotTopSorted | kAccessible | kNotAccessible |
                kCoAccessible | kNotCoAccessible |
@@ -227,25 +231,46 @@ uint64 ProjectProperties(uint64 inprops, bool project_input) {
   return outprops;
 }
 
+// Properties for a randgen FST.
+uint64 RandGenProperties(uint64 inprops, bool weighted) {
+  uint64 outprops = kAcyclic | kInitialAcyclic | kAccessible;
+  outprops |= inprops & kError;
+  if (weighted) {
+    outprops |= kTopSorted;
+    outprops |= (kAcceptor | kNoEpsilons |
+                 kNoIEpsilons | kNoOEpsilons |
+                 kIDeterministic | kODeterministic |
+                 kILabelSorted | kOLabelSorted) & inprops;
+  } else {
+    outprops |= kUnweighted;
+    outprops |= (kAcceptor | kILabelSorted | kOLabelSorted) & inprops;
+  }
+  return outprops;
+}
+
 // Properties for a replace FST.
 uint64 ReplaceProperties(const vector<uint64>& inprops,
                          ssize_t root,
-                         bool epsilon_on_replace,
+                         bool epsilon_on_call,
+                         bool epsilon_on_return,
+                         bool replace_transducer,
                          bool no_empty_fsts) {
   if (inprops.size() == 0)
     return kNullProperties;
   uint64 outprops = 0;
-  uint64 access_props = no_empty_fsts ? kAccessible || kCoAccessible : 0;
   for (size_t i = 0; i < inprops.size(); ++i)
-    access_props &= (inprops[i] & (kAccessible || kCoAccessible));
-  if (access_props == kAccessible || kCoAccessible) {
+    outprops |= kError & inprops[i];
+  uint64 access_props = no_empty_fsts ? kAccessible | kCoAccessible : 0;
+  for (size_t i = 0; i < inprops.size(); ++i)
+    access_props &= (inprops[i] & (kAccessible | kCoAccessible));
+  if (access_props == (kAccessible | kCoAccessible)) {
     outprops |= access_props;
     if (inprops[root] & kInitialCyclic)
       outprops |= kInitialCyclic;
     uint64 props =  0;
     bool string = true;
     for (size_t i = 0; i < inprops.size(); ++i) {
-      if (epsilon_on_replace == false)
+      if (replace_transducer)
         props |= kNotAcceptor & inprops[i];
       props |= (kNonIDeterministic | kNonODeterministic | kEpsilons |
                 kIEpsilons | kOEpsilons | kWeighted | kCyclic |
@@ -257,9 +282,9 @@ uint64 ReplaceProperties(const vector<uint64>& inprops,
     if (string)
       outprops |= kString;
   }
-  bool acceptor = epsilon_on_replace;
-  bool ideterministic = !epsilon_on_replace;
-  bool no_iepsilons = !epsilon_on_replace;
+  bool acceptor = !replace_transducer;
+  bool ideterministic = !epsilon_on_call && epsilon_on_return;
+  bool no_iepsilons = !epsilon_on_call && !epsilon_on_return;
   bool acyclic = true;
   bool unweighted = true;
   for (size_t i = 0; i < inprops.size(); ++i) {
@@ -273,6 +298,8 @@ uint64 ReplaceProperties(const vector<uint64>& inprops,
       acyclic = false;
     if (!(inprops[i] & kUnweighted))
       unweighted = false;
+    if (i != root && !(inprops[i] & kNoIEpsilons))
+      ideterministic = false;
   }
   if (acceptor)
     outprops |= kAcceptor;
@@ -291,7 +318,7 @@ uint64 ReplaceProperties(const vector<uint64>& inprops,
 
 // Properties for a relabeled FST.
 uint64 RelabelProperties(uint64 inprops) {
-  uint64 outprops = (kExpanded | kMutable |
+  uint64 outprops = (kExpanded | kMutable | kError |
                      kWeighted | kUnweighted |
                      kCyclic | kAcyclic |
                      kInitialCyclic | kInitialAcyclic |
@@ -303,11 +330,12 @@ uint64 RelabelProperties(uint64 inprops) {
 }
 
 // Properties for a reversed FST. (the superinitial state limits this set)
-uint64 ReverseProperties(uint64 inprops) {
+uint64 ReverseProperties(uint64 inprops, bool has_superinitial) {
   uint64 outprops =
-    (kExpanded | kMutable | kAcceptor | kNotAcceptor | kEpsilons |
-     kIEpsilons | kOEpsilons | kWeighted | kUnweighted |
-     kCyclic | kAcyclic) & inprops;
+    (kExpanded | kMutable | kError | kAcceptor | kNotAcceptor | kEpsilons |
+     kIEpsilons | kOEpsilons | kUnweighted | kCyclic | kAcyclic) & inprops;
+  if (has_superinitial)
+    outprops |= kWeighted & inprops;
   return outprops;
 }
 
@@ -321,7 +349,7 @@ uint64 ReweightProperties(uint64 inprops) {
 // Properties for an epsilon-removed FST.
 uint64 RmEpsilonProperties(uint64 inprops, bool delayed) {
   uint64 outprops = kNoEpsilons;
-  outprops |= (kAcceptor | kAcyclic | kInitialAcyclic) & inprops;
+  outprops |= (kError | kAcceptor | kAcyclic | kInitialAcyclic) & inprops;
   if (inprops & kAcceptor)
     outprops |= kNoIEpsilons | kNoOEpsilons;
   if (!delayed) {
@@ -342,8 +370,8 @@ uint64 ShortestPathProperties(uint64 props) {
 
 // Properties for a synchronized FST.
 uint64 SynchronizeProperties(uint64 inprops) {
-  uint64 outprops = (kAcceptor | kAcyclic | kAccessible | kCoAccessible |
-                     kUnweighted) & inprops;
+  uint64 outprops = (kError | kAcceptor | kAcyclic | kAccessible |
+                     kCoAccessible | kUnweighted) & inprops;
   if (inprops & kAccessible)
     outprops |= (kCyclic | kNotCoAccessible | kWeighted) & inprops;
   return outprops;
@@ -353,11 +381,14 @@ uint64 SynchronizeProperties(uint64 inprops) {
 uint64 UnionProperties(uint64 inprops1, uint64 inprops2, bool delayed) {
   uint64 outprops = (kAcceptor | kUnweighted | kAcyclic | kAccessible)
                     & inprops1 & inprops2;
+  outprops |= kError & (inprops1 | inprops2);
+  outprops |= kInitialAcyclic;
+
   bool empty1 = delayed;  // Can fst1 be the empty machine?
   bool empty2 = delayed;  // Can fst2 be the empty machine?
   if (!delayed) {
-    outprops |= (kExpanded | kMutable | kNotTopSorted | kNotString) & inprops1;
-    outprops |= (kNotTopSorted | kNotString) & inprops2;
+    outprops |= (kExpanded | kMutable | kNotTopSorted) & inprops1;
+    outprops |= kNotTopSorted & inprops2;
   }
   if (!empty1 && !empty2) {
     outprops |= kEpsilons | kIEpsilons | kOEpsilons;
@@ -381,7 +412,7 @@ uint64 UnionProperties(uint64 inprops1, uint64 inprops2, bool delayed) {
 // Property string names (indexed by bit position).
 const char *PropertyNames[] = {
   // binary
-  "expanded", "mutable", "", "", "", "", "", "",
+  "expanded", "mutable", "error", "", "", "", "", "",
   "", "", "", "", "", "", "", "",
   // trinary
   "acceptor", "not acceptor",
@@ -401,4 +432,4 @@ const char *PropertyNames[] = {
   "string", "not string",
 };
 
-}
+}  // namespace fst
