@@ -24,6 +24,7 @@ namespace script {
 
 REGISTER_FST_WEIGHT(StdArc::Weight);
 REGISTER_FST_WEIGHT(LogArc::Weight);
+REGISTER_FST_WEIGHT(Log64Arc::Weight);
 
 WeightClass::WeightClass(const string &weight_type,
                          const string &weight_str)
@@ -33,7 +34,7 @@ WeightClass::WeightClass(const string &weight_type,
   StrToWeightImplBaseT stw = reg->GetEntry(weight_type);
 
   impl_ = stw(weight_str, "WeightClass", 0);
-};
+}
 
 ostream& operator << (ostream &o, const WeightClass &c) {
   c.impl_->Print(&o);
